>From 8f92f0df0415dd3af7eba20290b1f9d8839e0325 Mon Sep 17 00:00:00 2001
From: Paul Fernquist <fernqui@hotmail.com>
Date: Fri, 26 Jun 2009 06:12:13 -0700
Subject: [PATCH 2/2] Fix SCTE20 CC  broken  in film 24fps to NTSC 30fps conversion

---
 modules/codec/cc.c             |   12 ++++--------
 modules/codec/cc.h             |   12 +++++++-----
 modules/codec/libmpeg2.c       |    6 ++++--
 modules/packetizer/h264.c      |    2 +-
 modules/packetizer/mpegvideo.c |    2 +-
 5 files changed, 17 insertions(+), 17 deletions(-)

diff --git a/modules/codec/cc.c b/modules/codec/cc.c
index 59614a1..c1922f9 100644
--- a/modules/codec/cc.c
+++ b/modules/codec/cc.c
@@ -381,6 +381,7 @@ static subpicture_t *Convert( decoder_t *p_dec, block_t *p_block )
     /* TODO do the real decoding here */
     while( p_block->i_buffer >= 3 )
     {
+ //       printf( "CCCCCCCCCCCC %02x  %02x %02x  %04x\n", p_block->p_buffer[0], p_block->p_buffer[1], p_block->p_buffer[2], p_sys->i_field);
         if( p_block->p_buffer[0] == p_sys->i_field )
             b_changed |= Eia608Parse( &p_sys->eia608, p_sys->i_channel, &p_block->p_buffer[1] );
 
@@ -617,16 +618,11 @@ static void Eia608ParseChannel( eia608_t *h, uint8_t d[2] )
     }
 
     /* */
-    const int d1 = d[0] & 0x7f;
-    const int d2 = d[1] & 0x7f;
-    if( d1 == 0x14 )
+    const int d1 = d[0] & 0x78;
+    if( d1 == 0x10 )
         h->i_channel = 1;
-    else if( d1 == 0x1c )
+    else if( d1 == 0x18 )
         h->i_channel = 2;
-    else if( d1 == 0x15 )
-        h->i_channel = 3;
-    else if( d1 == 0x1d )
-        h->i_channel = 4;
 }
 static bool Eia608ParseTextAttribute( eia608_t *h, uint8_t d2 )
 {
diff --git a/modules/codec/cc.h b/modules/codec/cc.h
index cb3740f..3af4ffc 100644
--- a/modules/codec/cc.h
+++ b/modules/codec/cc.h
@@ -77,7 +77,7 @@ static inline void cc_AppendData( cc_data_t *c, int i_field, const uint8_t cc[2]
     c->p_data[c->i_data++] = cc[1];
 }
 
-static inline void cc_Extract( cc_data_t *c, const uint8_t *p_src, int i_src )
+static inline void cc_Extract( cc_data_t *c, const uint8_t *p_src, int i_src, bool b_cc_fields_reversed )
 {
     static const uint8_t p_cc_ga94[4] = { 0x47, 0x41, 0x39, 0x34 };
     static const uint8_t p_cc_dvd[4] = { 0x43, 0x43, 0x01, 0xf8 };
@@ -201,7 +201,7 @@ static inline void cc_Extract( cc_data_t *c, const uint8_t *p_src, int i_src )
         for( int i = 0; i < i_cc_count; i++ )
         {
             bs_skip( &s, 2 );
-            const int i_field_idx = bs_read( &s, 2 );
+            int i_field_idx = bs_read( &s, 2 );
             bs_skip( &s, 5 );
             uint8_t cc[2];
             for( int j = 0; j < 2; j++ )
@@ -212,12 +212,14 @@ static inline void cc_Extract( cc_data_t *c, const uint8_t *p_src, int i_src )
             }
             bs_skip( &s, 1 );
 
-            if( i_field_idx != 1 && i_field_idx != 2 )
+            if( i_field_idx < 1 )
                 continue;
             if( c->i_data + 2*3 > CC_MAX_DATA_SIZE )
                 continue;
-
-            const int i_field = i_field_idx - 1;
+            /* In film mode(3 2 pull down), a third picture field(idx 3) is the repeat of the first field and
+             * an odd number of fields reverses field order of the following pictures */
+            int i_field = (i_field_idx - 1) & 1;
+            if(b_cc_fields_reversed) i_field ^= 1;
 
             cc_AppendData( c, i_field, &cc[0] );
         }
diff --git a/modules/codec/libmpeg2.c b/modules/codec/libmpeg2.c
index bd8eff6..41dc912 100644
--- a/modules/codec/libmpeg2.c
+++ b/modules/codec/libmpeg2.c
@@ -425,16 +425,18 @@ static picture_t *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
                     p_sys->i_cc_flags = BLOCK_FLAG_TYPE_B;
                 else p_sys->i_cc_flags = BLOCK_FLAG_TYPE_I;
 
+                bool b_cc_fields_reversed = !(p_sys->p_info->current_picture != NULL ?  p_sys->p_info->current_picture->flags & PIC_FLAG_TOP_FIELD_FIRST : 1);
+
                 if( p_sys->i_gop_user_data > 2 )
                 {
                     /* We now have picture info for any cached user_data out of the gop */
-                    cc_Extract( &p_sys->cc, &p_sys->p_gop_user_data[0], p_sys->i_gop_user_data );
+                    cc_Extract( &p_sys->cc, &p_sys->p_gop_user_data[0], p_sys->i_gop_user_data, b_cc_fields_reversed );
                     p_sys->i_gop_user_data = 0;
                 }
 
                 /* Extract the CC from the user_data of the picture */
                 if( p_info->user_data_len > 2 )
-                    cc_Extract( &p_sys->cc, &p_info->user_data[0], p_info->user_data_len );
+                    cc_Extract( &p_sys->cc, &p_info->user_data[0], p_info->user_data_len, b_cc_fields_reversed );
             }
         }
         break;
diff --git a/modules/packetizer/h264.c b/modules/packetizer/h264.c
index 3baac0c..ca8a167 100644
--- a/modules/packetizer/h264.c
+++ b/modules/packetizer/h264.c
@@ -1122,7 +1122,7 @@ static void ParseSei( decoder_t *p_dec, block_t *p_frag )
             if( i_t35 >= 5 &&
                 !memcmp( p_t35, p_dvb1_data_start_code, sizeof(p_dvb1_data_start_code) ) )
             {
-                cc_Extract( &p_sys->cc_next, &p_t35[3], i_t35 - 3 );
+                cc_Extract( &p_sys->cc_next, &p_t35[3], i_t35 - 3, false );
             }
         }
         i_used += i_size;
diff --git a/modules/packetizer/mpegvideo.c b/modules/packetizer/mpegvideo.c
index fa3c0e2..279e61a 100644
--- a/modules/packetizer/mpegvideo.c
+++ b/modules/packetizer/mpegvideo.c
@@ -625,7 +625,7 @@ static block_t *ParseMPEGBlock( decoder_t *p_dec, block_t *p_frag )
     }
     else if( p_frag->p_buffer[3] == 0xb2 && p_frag->i_buffer > 4 )
     {
-        cc_Extract( &p_sys->cc, &p_frag->p_buffer[4], p_frag->i_buffer - 4 );
+        cc_Extract( &p_sys->cc, &p_frag->p_buffer[4], p_frag->i_buffer - 4, false );
     }
     else if( p_frag->p_buffer[3] == 0x00 )
     {
-- 
1.6.1.2

